--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r118
Project Date		: 20081030164922

File				: SKG_Events.lua
Commit Author		: zhinjio
Commit Revision		: 118
Commit Date			: 20081030164922
************************************************************************
Description	:
	Event handling

TODO		:
	Need to find a replacement for FullyInitialized

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

-- All Event Listeners to go here. Everything else should be loaded at this point

-- I don't know if this has an ace3 equivalent. Not sure where I'm going to stuffthis code now
-- SKG:AddEventListener( "LibRockEvent-1.0", "FullyInitialized" )

-- Setup filters on inbound/outbound whispers for SKG stuff
-- Note, if you're running WIM, this won't work (yet)
function SKG_WhisperFilter( msg )
	-- inbound stuff
	-- These are safe to screen out
	if ( ( msg == "bid" ) or
		 ( msg == "retract" ) or
		 ( msg == "suicide" ) or
		 ( msg == "skg" ) ) then
		return true
	end
	-- outbound stuff
	-- don't do this. Requestor will not see any responses
end
ChatFrame_AddMessageEventFilter( "CHAT_MSG_WHISPER", SKG_WhisperFilter )

function SKG:CHAT_MSG_WHISPER( eventName, arg1, sender )
	-- This is here for backwards compatibility to the old way of
	-- doing things, and is necessary if the users in the raid are
	-- not running the new version of the addon at all
	local cmd = strlower( arg1 )
	if ( cmd == "bid" ) then
		-- bid handling code
		SKG_Debug( "Received bid request from " .. sender )
		if ( SKG.CurrBidList == "" ) then
			-- I have no idea there's a bidding event going on. Msg him back
			SendChatMessage( SKGL["SKG_ERROR_NOBIDEVENT"], "WHISPER", nil, sender )
		else
			-- We think there's a bid going on ...
			-- Are we in the same raid?
			if ( #SKG.Raiders ~= 0 ) then
				-- we're in a raid, is the sender in our raid?
				local found = 0
				for i = 1, #SKG.Raiders do
					if ( SKG.Raiders[i].name == sender ) then
						-- he's in our raid
						found = i
						break
					end
				end
				if ( found == 0 ) then
					SendChatMessage( SKGL["SKG_ERROR_NOTINMYRAID"], "WHISPER", nil, sender )
				else
					local inList = 0
					local db = SKG.db.profile.RollLists[SKG.CurrBidList]
					for i = 1, #db do
						if ( db[i] == SKG.Raiders[found].mPID ) then
							inList = i
							break
						end
					end
					if ( inList == 0 ) then
						SendChatMessage( SKGL["SKG_ERROR_NOTINBIDLIST"], "WHISPER", nil, sender )
						SKG_Print( SKGL["SKG_ERROR_INVALIDBIDDER"] .. " " .. sender )
					else
						-- Everything looks good. Send the bid event
						SKG_SendComm( "BIDBD", sender, "RAID", nil )
					end
				end
			end
		end
	elseif ( cmd == "suicide" ) then
		-- report back the current list
		SKG_Debug( "Received suicide request from " .. sender )
		-- New format:
		--		Your current list positions:
		--		"Main Raid" - positon 103 (6th in the current raid)
		--		"Alt Runs" - position 63 (3rd in the current raid)

		-- Find our sender
		-- if we're in a raid currently, and they are too, I've already got all
		-- the info I need
		local sPID = 0
		local InOurRaid = false
		local db = SKG.db.profile
		if ( SKG.inRaid == true ) then
			for i = 1, #SKG.Raiders do
				if ( SKG.Raiders[i].name == sender ) then
					sPID = SKG.Raiders[i].mPID
					InOurRaid = true
					break
				end
			end
		end
		if ( sPID == 0 ) then
			-- we didn't find him in the raid list, or we're not in a raid.
			-- do it the hard way
			for k, v in pairs( db.Players ) do
				if ( sender == v.name ) then
					sPID = k
					break
				end
			end
			if ( sPID == 0 ) then
				-- how about alts?
				for k, v in pairs( db.Alts ) do
					for i = 1, #v do
						if ( sender == v[i].name ) then
							sPID = k
							break
						end
					end
					if ( sPID ~= 0 ) then
						break
					end
				end
			end
		end
		-- if we still couldn't find them at all, just let them know they're
		-- not in any roll lists currently
		if ( sPID == 0 ) then
			SendChatMessage( SKGL["SKG_ERROR_NOTINLISTS"], "WHISPER", nil, sender )
			return
		end

		local outT = {}
		-- If we made it here, we found them. Proceed.
		for k, v in pairs( db.RollLists ) do
			-- loop through list
			local currentraid = 0
			for i = 1, #v do
				if ( ( SKG.inRaid ) and ( InOurRaid == true ) ) then
					-- we need to increment a "currentraid" counter as well
					for j = 1, #SKG.Raiders do
						if ( v[i] == SKG.Raiders[j].mPID ) then
							SKG_Debug( "ir " .. v[i] .. "/" .. SKG.Raiders[j].mPID )
							SKG_Debug( "sPID " .. sPID )
							if ( v[i] ~= sPID ) then
								currentraid = currentraid + 1
								SKG_Debug( "cr+1 " .. currentraid )
								break
							end
						end
					end
				end
				if ( v[i] == sPID ) then
					local t = { i, currentraid + 1 }
					outT[k] = t
					SKG_Debug( "k, t1, t2 = " .. k .. "/" .. i .. "/" .. (currentraid+1) )
					break
				end
			end
		end
		if ( next(outT) == nil ) then
			SendChatMessage( SKGL["SKG_ERROR_NOTINLISTS"], "WHISPER", nil, sender )
		else
			SendChatMessage( SKGL["SKG_MSG_LISTPOSHDR"], "WHISPER", nil, sender )
			for k, v in pairs( outT ) do
				if ( InOurRaid == true ) then
					SendChatMessage( string.format( SKGL["SKG_MSG_LISTPOSITEMR"], k, v[1], v[2] ),
						"WHISPER", nil, sender )
				else
					SendChatMessage( string.format( SKGL["SKG_MSG_LISTPOSITEM"], k, v[1] ),
						"WHISPER", nil, sender )
				end
			end
			if ( InOurRaid == false ) then
				SendChatMessage( SKGL["SKG_MSG_NOTINMYRAID"], "WHISPER", nil, sender )
			end
		end
	elseif ( cmd == "retract" ) then
		-- bid retraction code
		SKG_Debug( "Received retract request from " .. sender )
		if ( SKG.CurrBidList == "" ) then
			-- I have no idea there's a bidding event going on. Msg him back
			SendChatMessage( SKGL["SKG_ERROR_NOBIDEVENT"], "WHISPER", nil, sender )
		else
			-- We think there's a bid going on ...
			-- Are we in the same raid?
			if ( #SKG.Raiders ~= 0 ) then
				-- we're in a raid, is the sender in our raid?
				local found = 0
				for i = 1, #SKG.Raiders do
					if ( SKG.Raiders[i].name == sender ) then
						-- he's in our raid
						found = i
						break
					end
				end
				if ( found == 0 ) then
					SendChatMessage( SKGL["SKG_ERROR_NOTINMYRAID"], "WHISPER", nil, sender )
				else
					local inList = 0
					local db = SKG.db.profile.RollLists[SKG.CurrBidList]
					for i = 1, #db do
						if ( db[i] == SKG.Raiders[found].mPID ) then
							inList = i
							break
						end
					end
					if ( inList == 0 ) then
						SendChatMessage( SKGL["SKG_ERROR_NOTINBIDLIST"], "WHISPER", nil, sender )
						SKG_Print( SKGL["SKG_ERROR_INVALIDBIDDER"] .. " " .. sender )
					else
						-- Everything looks good. Send the bid event
						SKG_SendComm( "BIDRT", sender, "RAID", nil )
					end
				end
			end
		end
		return
	elseif ( cmd == "skg" ) then
		SendChatMessage( SKGL["SKG_MSG_SKGINST1"], "WHISPER", nil, sender )
		SendChatMessage( SKGL["SKG_MSG_SKGINST2"], "WHISPER", nil, sender )
		SendChatMessage( SKGL["SKG_MSG_SKGINST3"], "WHISPER", nil, sender )
	end
end

function SKG:RAID_ROSTER_UPDATE( )
	SKG_Debug( "Received RAID_ROSTER_UPDATE" )
	local ActiveList = SKG_ActiveListTitle:GetText()
	local numRaiders = GetNumRaidMembers()
	if ( numRaiders > 0 ) then
		-- toggle main variables
		SKG.inRaid = true
		SKG_UpdateRaiders()

		SKG_LF_SpamListButton:Enable()
		-- if "raidonly" is set, disable some of the listmgmt buttons
		if ( ( SKG.db.profile.Options.General.raidonly ) and
			 ( ActiveList ~= nil ) and
			 ( ActiveList ~= "Players" ) ) then
			-- Reserve will "re-enable" if they select a reserved player
			SKG_LF_ReservePlayerButton:Disable()
			SKG_LF_MoveDownButton:Disable()
			SKG_LF_MoveUpButton:Disable()
		end

		-- if you also happen to be raid leader, enable these:
		if ( IsRaidLeader() == 1 ) then
			SKG_BF_LootTypeButton:Show()
			SKG_BF_LootThreshButton:Show()
			SKG_BF_LootTypeButton:Enable()
			SKG_BF_LootThreshButton:Enable()
			-- and fill in the ML assignment dewdrop
			SKG.lootTypeSelections.args[3].args = {}
			for i = 1, #SKG.Raiders do
				raidName = SKG.Raiders[i].name
				raidIndex = SKG.Raiders[i].raidID
				SKG_Debug( "n = " .. raidName .. ", rI = " .. raidIndex )
				local a = {
					type = "execute",
					name = raidName,
					desc = raidName,
					func = function( value )
						-- Set Master Looter to this player
						SetLootMethod( "master", value )
						Dewdrop:Close()
					end,
					passValue = raidName,
					order = raidIndex,
				}
				table.insert( SKG.lootTypeSelections.args[3].args, a )
			end
		else
			SKG_BF_LootTypeButton:Hide()
			SKG_BF_LootThreshButton:Hide()
		end
	else
		SKG.inRaid = false
		SKG.Raiders = {}
		-- hide buttons/panels, etc
		SKG_LF_SpamListButton:Disable()
		if ( SKG_FRAME_STATE == "Bid" ) then
			SKG_BidButtonFrame:Hide()
		end
		SKG_BF_LootTypeButton:Disable()
		SKG_BF_LootThreshButton:Disable()
	
		-- if "raidonly" is set, disable some of the listmgmt buttons
		if ( ( ActiveList ~= nil ) and
			 ( ActiveList ~= "Players" ) ) then
			-- Reserve will "re-enable" if they select a reserved player
			SKG_LF_ReservePlayerButton:Enable()
			SKG_LF_MoveDownButton:Enable()
			SKG_LF_MoveUpButton:Enable()
		end
	end
	SKG_UpdateCurrentList()
end

function SKG:ADDON_LOADED( eventName, arg1 )
	if ( arg1 == "SKG" ) then
		local db = SKG.db.profile
		SKG_Debug( "Processing ADDON_LOADED event for SKG" )
		-- We're safe to do things like process saved variables here.
		-- See the header of SKG_Globals.lua for a full reference of
		-- possible values

		-- Call InitAllPanels to make sure we have the right values
		SKG_InitAllPanels()

		-- Testing out comms...
		SKG_SendComm( "DEBUG", "test message", "GUILD", nil )
	end
end

function SKG:UNIT_SPELLCAST_SENT( eventName, spellCaster, spellName, blort, spellTarget )
	-- I watch this because occasionally a loot event will be the opening of a chest
	-- or something else that does not translate into a mob with a GUID.
	--
	-- So if this spell being cast is "Opening", and the caster is "player", then
	-- save the target data so I reference it later in the LOOT OPENED event

	-- NOTE TO FIX FIX FIX FIX : I believe this will currently only work
	-- for english clients. need to localize or something. Further, there
	-- are apparently other spells that open things. So I'll need to find those
	-- too eventually
	if ( ( spellCaster == "player" ) and ( spellName == "Opening" ) ) then
		SKG.OpenedChest = spellTarget
		SKG_Debug( "caster/spell/tgt = " .. spellCaster .. "/" .. spellName .. "/" .. spellTarget )
	end
	if ( ( spellCaster == "player" ) and 
		( ( spellName == "Disenchant" ) or 
		  ( spellName == "Herb Gathering" ) or
		  ( spellName == "Mining" ) ) ) then
		SKG.DE = true
		SKG_Debug( "player just DE/Mined/Herb'd an item ... ignoring" )
	end
end

function SKG:LOOT_OPENED( )
	-- The player who is the master looter should get the goods
	SKG_Debug( "Received LOOT_OPENED event" )

	-- Note: This ASSUMES that looting something forces you to target the mob you're looting
	local mobGUID = UnitGUID("target")
	local mobName = UnitName("target")

	if ( ( mobGUID == nil ) or ( mobGUID == "" ) or ( mobName == "" ) ) then
		mobGUID = 0
		if ( SKG.OpenedChest ~= "" ) then
			mobName = SKG.OpenedChest
			SKG.OpenedChest = ""
		else
			mobName = "Unknown"
		end
	end

	-- If the player just DE'd something, ignore
	if ( SKG.DE == true ) then
		SKG.DE = false
		return
	end
			
	-- Only process anything if we're in a raid
	if not ( GetNumRaidMembers() == 0 ) then
		-- we're in a raid
		-- Only do stuff if we're the Loot Master
		SKG_Debug( "We're in a raid" )
		local myRaidID = SKG_GetMyRaidID()
		local lootMethod, MLPartyID, MLRaidID = GetLootMethod()
		if ( lootMethod == "master" ) then
			SKG_Debug( "Master Looter method" )
			if ( MLRaidID == myRaidID ) then
				-- we're the ML, do stuff
				SKG_Debug( "I'm the ML" )
				-- Send a LOOTC event so everyone's loot lists get cleared
				-- Actually, I'm gonna remove this. People should be clearing out
				-- items that they don't want handled in a loot event by hand.
				-- SKG:SendCommMessage( "GROUP", "LOOTC", "" )

				-- Iterate through the looted items, see which ones match out threshold
				local LootThreshold = GetLootThreshold()
				SKG_Debug( "threshold = " .. LootThreshold )
				local index = 1
				for index = 1, GetNumLootItems(), 1 do
					if ( LootSlotIsItem(index) ) then
						local itemLink = GetLootSlotLink( index );
						local itemID = string.match( itemLink, "item:(%d+)" ) + 0
						local _, _, itemRarity = GetItemInfo( itemLink )
						SKG_Debug( "itemID is " .. itemID ) 
						if ( SKG_LootIgnore[itemID] ) then
							SKG_Print( string.format( SKGL["SKG_MSG_LOOTIGNORED"], itemLink ) )
							-- this is an item we're ignoring. Don't do anything
						else
							-- check our loot threshold
							if ( itemRarity >= LootThreshold ) then
								-- One final check. Is the item in question already in our
								-- biddable items list:
								local found = false
								for i = 1, #SKG.LootItems do
									if ( mobGUID == SKG.LootItems[i].mGUID ) then
										SKG_Debug( "our mobguids match" )
										if ( SKG_SmartCompareItemlinks(
												SKG.LootItems[i].link, itemLink ) == true ) then
											SKG_Debug( "full match" )
											found = true
											break
										end
									end
								end

								if ( found == false ) then
									local payload = { mobGUID, mobName, itemLink }
									SKG_SendComm( "LOOTI", payload, "RAID", nil )
								else
									-- do nothing. We already have this item in our loot table
								end
							end
						end
					end
				end
			else
				SKG_Debug( "I'm not the ML, myrid = " .. myRaidID .. ", value = " .. MLRaidID )
			end
		else
			SKG_Debug( "Master Looter isn't the method, " .. lootMethod )
		end
	else
		SKG_Debug( "We're not in a raid" )
	end
end

function SKG:GUILD_ROSTER_UPDATE( )
-- SKG.FullyInitialized = function( self, value, namespace, event )
	if ( SKG_INIT_GUILD == 0 ) then
		SKG_Debug( "GRU, haven't initted yet. This shouldn't happen" )
	elseif ( SKG_INIT_GUILD == 1 ) then
		SKG_Debug( "GRU, globals done, I've run init. I'm gonna wait for one more." )
		SKG_INIT_GUILD = 2
	elseif ( SKG_INIT_GUILD == 2 ) then
		-- Lets try this instead of FullyInitialized
		SKG_Debug( "Received GUILD_ROSTER_UPDATE event" )

		-- Some values just don't seem to be present even after all the regular addon
		-- processing has occurred. So lets try this, after the user is already in, and
		-- has opted to open the main frame...

		SKG_INIT_GUILD = 3

		SKG.me = UnitName( "player" )
		SKG_Debug( "me = " .. SKG.me )

		-- Pretty sure this is not going to be needed...
		SKG_SetSecurityLevel()
		local major, minor = SKG_getVersionComponents()
		SKG_TitleText:SetText( string.format( SKGL["SKG_LBL_TITLE_TEXT"] , 
			major, minor, SKG_SECURITY_LEVEL) )

		-- If we're the guild leader or the proxy, enable the admin module and init stuff
		if ( ( IsGuildLeader( SKG.me ) ) or
			 ( SKG_SECURITY_LEVEL == "P" ) ) then
			SKG_Debug( "About to toggle security active" )
			SKG:EnableModule( "SKGAdmin" )
			SKG_SF_InitValues()
		end

		-- We should be able to fill in the "mainChooser" selector for inserting players
		SKG_UpdateSPlayers()
		SKG_UpdateMainsChooser()
		SKG_UpdateToonChooser()

		-- Do the initial setup of SPlayers
		SKG_UpdateSPlayers()

		-- Attempt to detect whether the old version of SKG is loaded and if we possibly
		-- have lists to convert
		-- This should ONLY happen if you're a Syncer
		if ( ( SKG_SECURITY_LEVEL ~= "U" ) and ( SKG_Options ) ) then
			StaticPopup_Show( "SKG_CONFIRM_IMPORT" )
		else
			SKG_Debug("Old version not present")
		end

		-- Detect if an old version is loaded in addition. I want to completely disable
		-- if that is the case
		if ( SKG_ROLL_MIN ) then
			-- this was a variable used in 4.32 version. Print out a message and disable
			SKG_Print( SKGL["SKG_ERROR_OLDVERSION"] )
			SKG:Disable()
		end

		-- Since I was dumb and made a mistake with one of the ways I saved security
		-- policies, go ahead and clean up the mess if it exists.
		local dbSO = SKG.db.profile.Options.Security
		if ( dbSO.Policies ) then
			if ( dbSO.Policies.myServer ) then
				dbSO.Policies.myServer.myFaction.myGuild.keydata = nil
				dbSO.Policies.myServer.myFaction.myGuild.timestamp = nil
				dbSO.Policies.myServer.myFaction.myGuild = nil
				dbSO.Policies.myServer.myFaction = nil
				dbSO.Policies.myServer = nil
			end
		end

		-- Possible fix for the layout-cache.txt dewdrop behind frame problem
		SKG_Frame:SetUserPlaced(false)

		-- Send out a SYNPL to ping lists from others. We may not have all the current lists.
		SKG_SendComm( "SYNPL", {}, "GUILD", nil )
	else
		-- We can ignore now
		SKG:UnregisterEvent( "GUILD_ROSTER_UPDATE" )
	end
end

--[[
************************************************************************
CHANGELOG:

Date : 10/26/08
	Fixing Ace3 conversion errors. Moving RegisterEvent calls to OnInit
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer. Changed the function definitions
	to only use what arguments they really need/use. 
************************************************************************
]]--
